@@ -1,3 +1,7 @@
+
+#'
+#' @importFrom GDINA attributepattern
+#'
 correctQ.Hull <- function(Y, Q, CDM.obj=NULL, method="BM", mono.constraint=TRUE, model="GDINA",
                           search.method="ESA", maxitr=1, iter.level="test",
                           criter="PVAF", verbose = TRUE){
@@ -14,6 +18,9 @@ correctQ.Hull <- function(Y, Q, CDM.obj=NULL, method="BM", mono.constraint=TRUE,
     Q.pattern.ini[i] <- get.Pattern(Q.Hull[i, ], pattern)
   Q.pattern <- Q.pattern.ini
 
+  Hull.fit <- list()
+  class(Hull.fit) <- "Hull"
+  
   iter <- 0
   while(iter < maxitr){
     iter <- iter + 1
@@ -28,11 +35,15 @@ correctQ.Hull <- function(Y, Q, CDM.obj=NULL, method="BM", mono.constraint=TRUE,
 
     Q.pattern.cur <- rep(2, I)
     criterion.pre <- criterion.cur <- rep(0, I)
+    pattern.criterion <- rep(2, K)
     for(i in 1:I){
-        P.est <- (colSums(Y[, i] * P.alpha.Xi) + 1e-10) / (colSums(P.alpha.Xi) + 2e-10)
+      
+      P.est <- (colSums(Y[, i] * P.alpha.Xi) + 1e-10) / (colSums(P.alpha.Xi) + 2e-10)
+      
       if(criter == "PVAF"){
         P.mean <- sum(P.est * P.alpha)
-        zeta2.i.K <- sum((P.est - P.mean)^2 * P.alpha)
+        P.Xi.alpha.L <- P.GDINA(rep(1, K), P.est, pattern, P.alpha)
+        zeta2.i.K <- sum((P.Xi.alpha.L - P.mean)^2 * P.alpha)
 
         P.Xi.alpha <- P.GDINA(pattern[Q.pattern.ini[i], ], P.est, pattern, P.alpha)
         criterion.pre[i] <- criterion.cur[i] <- sum((P.Xi.alpha - P.mean)^2 * P.alpha) / zeta2.i.K
@@ -51,30 +62,37 @@ correctQ.Hull <- function(Y, Q, CDM.obj=NULL, method="BM", mono.constraint=TRUE,
 
         criterion.pre[i] <- criterion.cur[i] <- 1- L.X[Q.pattern.ini[i]] / L.X[1]
       }
+      
       ######################################## ESA ########################################
       if(search.method == "ESA"){
         if(criter == "PVAF"){
           zeta2 <- rep(-Inf, K)
           zeta2.cur <- 0
-          pattern.zeta2 <- rep(2, K)
           for(l in 2:L){
             P.Xi.alpha <- P.GDINA(pattern[l, ], P.est, pattern, P.alpha)
             zeta2.cur <- sum((P.Xi.alpha - P.mean)^2 * P.alpha)
             if(zeta2.cur > zeta2[sum(pattern[l, ])]){
               zeta2[sum(pattern[l, ])] <- zeta2.cur
-              pattern.zeta2[sum(pattern[l, ])] <- l
+              pattern.criterion[sum(pattern[l, ])] <- l
             }
           }
           criterion <- zeta2 / zeta2[K]
         }else if(criter == "R2"){
           P.est.temp <- matrix(P.est, N, L, byrow = TRUE)
+          R2 <- rep(-Inf, K)
+          R2.cur <- 0
           for(l in 2:L){
             P.Xi.alpha <- P.GDINA(pattern[l, ], P.est, pattern, P.alpha)
             P.Xi.alpha.temp <- matrix(P.Xi.alpha, N, L, byrow = TRUE)
             L.Xi <- rowSums(P.alpha.Xi*(P.Xi.alpha.temp^Y.temp)*((1-P.Xi.alpha.temp)^(1-Y.temp)))
             L.X[l] <- sum(log(L.Xi))
+            R2.cur <- 1- L.X[l] / L.X[1]
+            if(R2.cur > R2[sum(pattern[l, ])]){
+              R2[sum(pattern[l, ])] <- R2.cur
+              pattern.criterion[sum(pattern[l, ])] <- l
+            }
           }
-          criterion <- 1- L.X / L.X[1]
+          criterion <- R2
         }
 
         Hull <- rep(-Inf, K-1)
@@ -89,27 +107,95 @@ correctQ.Hull <- function(Y, Q, CDM.obj=NULL, method="BM", mono.constraint=TRUE,
           nex <- (fjk[k+2]-fjk[k+1]) / (npk[k+2]- npk[k+1])
           Hull[k] <- pre / nex
         }
-        Hull <- keep.convex(Hull, fjk, npk)
-        if(!any(is.nan(Hull))){
-          if(Q.pattern.cur[i] == 2 & all(Hull < 0)){
+        Hull.convex <- keep.convex(Hull, fjk, npk)
+        if(!any(is.nan(Hull.convex$Hull))){
+          if(Q.pattern.cur[i] == 2 & all(Hull.convex$Hull < 0)){
             Q.pattern.cur[i] <- L
           }else{
-            Q.pattern.cur[i] <- pattern.zeta2[which.max(Hull)]
+            Q.pattern.cur[i] <- pattern.criterion[which.max(Hull.convex$Hull)]
           }
         }
         criterion.cur[i] <- criterion[sum(pattern[Q.pattern.cur[i], ])]
+        
+        posi <- sort(unique(unlist(Hull.convex$posi)))
+        number.of.parameters <- c(0, 2^(1:K))
+        fit.index <- c(0, criterion)
+        Hull.fit[[i]] <- list(number.of.parameters=number.of.parameters, fit.index=fit.index, posi=posi, 
+                               pattern.criterion=pattern.criterion, pattern=pattern, 
+                               criter=criter, sug=criterion.cur[i])
+        
+      }
+        
+      ######################################## SSA ########################################
+      if(search.method == "SSA"){
 
-        # {
-        #   Hull[is.nan(Hull)] <- 0
-        #   number.of.parameters <- c(0, npk[c(which(Hull>0)+1, K+1)])
-        #   fit.index <- c(0, criterion[c(which(Hull>0)+1, K)])
-        #   plot(number.of.parameters, fit.index, type = "o", main = "Hull plot")
-        # }
-
+        Q.i <- criterion <- rep(0, K)
+        for(k in 1:K){
+          Q.i.k <- Q.i
+          for(kk in 1:K){
+            Q.i.cur <- Q.i
+            if(Q.i[kk] == 0){
+              Q.i.cur[kk] <- 1
+              q.possible.cur <- get.Pattern(Q.i.cur, pattern)
+              P.Xi.alpha.cur <- P.GDINA(Q.i.cur, P.est, pattern, P.alpha)
+              
+              if(criter == "PVAF"){
+                PVAF.i.k.cur <- sum((P.Xi.alpha.cur - P.mean)^2 * P.alpha) / zeta2.i.K
+                if(criterion[sum(Q.i.cur)] < PVAF.i.k.cur){
+                  Q.i.k <- Q.i.cur
+                  pattern.criterion[sum(Q.i.cur)] <- q.possible.cur
+                  criterion[sum(Q.i.cur)] <- PVAF.i.k.cur
+                }
+              }else if(criter == "R2"){
+                P.Xi.alpha.temp <- matrix(P.Xi.alpha.cur, N, L, byrow = TRUE)
+                L.Xi <- rowSums(P.alpha.Xi*(P.Xi.alpha.temp^Y.temp)*((1-P.Xi.alpha.temp)^(1-Y.temp)))
+                L.X.cur <- sum(log(L.Xi))
+                R2.i.k.cur <- 1- L.X.cur / L.X[1]
+                if(criterion[sum(Q.i.cur)] < R2.i.k.cur){
+                  Q.i.k <- Q.i.cur
+                  pattern.criterion[sum(Q.i.cur)] <- q.possible.cur
+                  criterion[sum(Q.i.cur)] <- R2.i.k.cur
+                }
+              }
+            }
+          }
+          Q.i <- Q.i.k
+        }
+        
+        Hull <- rep(-Inf, K-1)
+        fjk <- rep(0, K+1)
+        npk <- rep(0, K+1)
+        for(k in 1:K){
+          fjk[k+1] <- criterion[k]
+          npk[k+1] <- 2^k
+        }
+        for(k in 1:(K-1)){
+          pre <- (fjk[k+1]-fjk[k]) / (npk[k+1]- npk[k])
+          nex <- (fjk[k+2]-fjk[k+1]) / (npk[k+2]- npk[k+1])
+          Hull[k] <- pre / nex
+        }
+        Hull.convex <- keep.convex(Hull, fjk, npk)
+        if(!any(is.nan(Hull.convex$Hull))){
+          if(Q.pattern.cur[i] == 2 & all(Hull.convex$Hull < 0)){
+            Q.pattern.cur[i] <- L
+          }else{
+            Q.pattern.cur[i] <- pattern.criterion[which.max(Hull.convex$Hull)]
+          }
+        }
+        criterion.cur[i] <- criterion[sum(pattern[Q.pattern.cur[i], ])]
+        
+        posi <- sort(unique(unlist(Hull.convex$posi)))
+        number.of.parameters <- c(0, 2^(1:K))
+        fit.index <- c(0, criterion)
+        Hull.fit[[i]] <- list(number.of.parameters=number.of.parameters, fit.index=fit.index, posi=posi, 
+                               pattern.criterion=pattern.criterion, pattern=pattern, 
+                               criter=criter, sug=criterion.cur[i])
+        
       }
 
       ######################################## PAA ########################################
       if(search.method == "PAA"){
+        
         priority.cur <- get.MLRlasso(alpha.P, Y[, i])
         if(all(priority.cur <= 0))
           priority.cur[which.max(priority.cur)] <- 1
@@ -117,7 +203,7 @@ correctQ.Hull <- function(Y, Q, CDM.obj=NULL, method="BM", mono.constraint=TRUE,
 
         priority.temp <- priority.cur
         Q.i <- rep(0, K)
-        pattern.zeta2 <- c()
+        pattern.criterion <- c()
 
         search.length <- ifelse(length(which(priority.cur > 0)) < K, length(which(priority.cur > 0)) + 1, K)
         fjk <- npk <- Hull <- c(0)
@@ -128,7 +214,7 @@ correctQ.Hull <- function(Y, Q, CDM.obj=NULL, method="BM", mono.constraint=TRUE,
           q.possible.cur <- get.Pattern(Q.i.cur, pattern)
           priority.temp[att.posi] <- -Inf
 
-          pattern.zeta2[sum(Q.i.cur)] <- get.Pattern(Q.i.cur, pattern)
+          pattern.criterion[sum(Q.i.cur)] <- get.Pattern(Q.i.cur, pattern)
 
           if(criter == "PVAF"){
             P.Xi.alpha.cur <- P.GDINA(Q.i.cur, P.est, pattern, P.alpha)
@@ -138,37 +224,36 @@ correctQ.Hull <- function(Y, Q, CDM.obj=NULL, method="BM", mono.constraint=TRUE,
             P.Xi.alpha <- P.GDINA(Q.i.cur, P.est, pattern, P.alpha)
             P.Xi.alpha.temp <- matrix(P.Xi.alpha, N, L, byrow = TRUE)
             L.Xi <- rowSums(P.alpha.Xi*(P.Xi.alpha.temp^Y.temp)*((1-P.Xi.alpha.temp)^(1-Y.temp)))
-            L.X[pattern.zeta2[sum(Q.i.cur)]] <- sum(log(L.Xi))
-            fjk <- c(fjk, 1- L.X[pattern.zeta2[sum(Q.i.cur)]] / L.X[1])
+            L.X[pattern.criterion[sum(Q.i.cur)]] <- sum(log(L.Xi))
+            fjk <- c(fjk, 1- L.X[pattern.criterion[sum(Q.i.cur)]] / L.X[1])
           }
           npk <- c(npk, 2^sum(Q.i.cur))
           Q.i <- Q.i.cur
         }
+        
         for(k in 1:(search.length-1)){
           pre <- (fjk[k+1]-fjk[k]) / (npk[k+1]- npk[k])
           nex <- (fjk[k+2]-fjk[k+1]) / (npk[k+2]- npk[k+1])
           Hull[k] <- pre / nex
         }
 
-        if(length(pattern.zeta2) == 1){
-          Q.pattern.cur[i] <- pattern.zeta2
-        }else if(length(pattern.zeta2) == 2){
-          if(max(Hull) > 1){
-            Q.pattern.cur[i] <- pattern.zeta2[which.max(Hull)]
+        Hull.convex <- keep.convex(Hull, fjk, npk)
+        if(!any(is.nan(Hull.convex$Hull))){
+          if(all(Hull.convex$Hull < 0)){
+            Q.pattern.cur[i] <- pattern.criterion[length(pattern.criterion)]
           }else{
-            Q.pattern.cur[i] <- pattern.zeta2[2]
-          }
-        }else{
-          Hull <- keep.convex(Hull, fjk, npk)
-          if(!any(is.nan(Hull))){
-            if(Q.pattern.cur[i] == 2 & all(Hull < 0)){
-              Q.pattern.cur[i] <- L
-            }else{
-              Q.pattern.cur[i] <- pattern.zeta2[which.max(Hull)]
-            }
+            Q.pattern.cur[i] <- pattern.criterion[which.max(Hull.convex$Hull)]
           }
         }
-        criterion.cur[i] <- fjk[which.max(Hull) + 1]
+        criterion.cur[i] <- fjk[which(pattern.criterion == Q.pattern.cur[i]) + 1]
+        
+        posi <- sort(unique(unlist(Hull.convex$posi)))
+        number.of.parameters <- c(0, 2^(1:length(pattern.criterion)))
+        fit.index <- fjk
+        Hull.fit[[i]] <- list(number.of.parameters=number.of.parameters, fit.index=fit.index, posi=posi, 
+                               pattern.criterion=pattern.criterion, pattern=pattern, 
+                               criter=criter, sug=criterion.cur[i])
+        
       }
     }
 
@@ -207,7 +292,7 @@ correctQ.Hull <- function(Y, Q, CDM.obj=NULL, method="BM", mono.constraint=TRUE,
     if(verbose){
       cat(paste0('Iter=', iter, "/", maxitr, ","),
           change, 'items have changed,',
-          paste0(paste0("delta ", criter, "="), formatC(sum(criterion.delta), digits = 5, format = "f")), "\n")
+          paste0(paste0("\u0394 ", criter, "="), formatC(sum(criterion.delta), digits = 5, format = "f")), "\n")
     }
   }
   if(search.method == "PAA"){
@@ -215,6 +300,8 @@ correctQ.Hull <- function(Y, Q, CDM.obj=NULL, method="BM", mono.constraint=TRUE,
     colnames(priority) <- colnames(Q)
   }
 
-  return(list(Q.original = Q, Q.sug = Q.Hull, priority=priority, iter = iter - 1))
+  return(list(Q.original = Q, Q.sug = Q.Hull, priority=priority, 
+              Hull.fit = Hull.fit, 
+              iter = iter - 1))
 
 }
